/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import antlr.RecognitionException;
import antlr.TokenStreamException;
import antlr.collections.AST;
import de.bsvrz.ibv.uda.client.ClientSkript;
import de.bsvrz.ibv.uda.client.LokalesSkript;
import de.bsvrz.ibv.uda.interpreter.anweisung.Anweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.AusgabeAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.AusnahmeAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.BedingterAnweisungsBlock;
import de.bsvrz.ibv.uda.interpreter.anweisung.BenutzeAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.MitFehlerAusgabeAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.MitFehlerRueckgabeAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.OhneFehlerPruefungAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.RueckgabeAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.VersucheAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.WennAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.ZuweisungsAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.ZyklischeAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.AnmeldeAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.AnmeldeAusdruck;
import de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.AtgAnmeldungsEintragAusdruck;
import de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.MengenAnmeldungsEintragAusdruck;
import de.bsvrz.ibv.uda.interpreter.anweisung.iteration.BedingungsIterator;
import de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsAnweisung;
import de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsBedingung;
import de.bsvrz.ibv.uda.interpreter.anweisung.iteration.LaufIterationsBedingung;
import de.bsvrz.ibv.uda.interpreter.anweisung.iteration.MengenIterationsBedingung;
import de.bsvrz.ibv.uda.interpreter.ausdruck.ElementZugriffsSymbol;
import de.bsvrz.ibv.uda.interpreter.ausdruck.FeldInitialisiererSymbol;
import de.bsvrz.ibv.uda.interpreter.ausdruck.FunktionsAufruf;
import de.bsvrz.ibv.uda.interpreter.ausdruck.FunktionsDefinition;
import de.bsvrz.ibv.uda.interpreter.ausdruck.ListenInitialisiererSymbol;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Quantor;
import de.bsvrz.ibv.uda.interpreter.ausdruck.QuantorBedingung;
import de.bsvrz.ibv.uda.interpreter.ausdruck.QuantorDefinition;
import de.bsvrz.ibv.uda.interpreter.ausdruck.UdaBasisAusdruck;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Variable;
import de.bsvrz.ibv.uda.interpreter.daten.container.ContainerMehrfachSelektionsSymbol;
import de.bsvrz.ibv.uda.interpreter.daten.container.ContainerSelektorSymbol;
import de.bsvrz.ibv.uda.interpreter.daten.container.ContainerZugriffsSymbol;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.BooleanLiteral;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.FliessKommaZahlLiteral;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.GanzZahlLiteral;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.RelativeZeitLiteral;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.ZahlenLiteral;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.ZeichenkettenLiteral;
import de.bsvrz.ibv.uda.parser.CommonASTWithLines;
import de.bsvrz.ibv.uda.parser.UDALexer;
import de.bsvrz.ibv.uda.parser.UDAParser;
import de.bsvrz.ibv.uda.parser.UDATokensTokenTypes;
import de.bsvrz.ibv.uda.uda.UdaMessage;
import de.bsvrz.ibv.uda.uda.data.Skript;
import de.bsvrz.ibv.uda.verwaltung.ServerSkript;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operation;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse zur Umsetzung des Quelltextes eines Uda-Skripts in eine von einem
 * {@link UdaInterpreter} ausführbare Form.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public final class InterpreterFactory extends UDAParser {

	/** Logger für Debugausgaben. */
	private static final Debug LOGGER = de.bsvrz.sys.funclib.debug.Debug.getLogger();

	/**
	 * Menge der analysierten Skripte: wird benötigt, um mehrfaches Einbinden
	 * eines Skripts zu verhindern.
	 */
	private static final Set<Skript> ANALYSIERT = new HashSet<>();

	/**
	 * erzeugt den Ausdruck für eine Anmeldeanweisung.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt im vom Parser gelieferten Syntaxbaum
	 * @param skript
	 *            der Name des Skripts
	 * @return die Anweisung
	 */
	private static Anweisung createAnmeldeAnweisung(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		final AST nameAst = ast.getFirstChild();
		final AnmeldeAnweisung anweisung = new AnmeldeAnweisung(InterpreterFactory.getLine(ast), skript,
				nameAst.getText());
		AnweisungsBlock anweisungen = null;

		AST abschnitt = nameAst.getNextSibling();
		while (abschnitt != null) {
			switch (abschnitt.getType()) {
			case UDATokensTokenTypes.Quelle:
				Collection<AnmeldeAusdruck> liste = InterpreterFactory.createAnmeldungsListe(interpreter, abschnitt,
						skript);
				anweisung.setQuellen(liste);
				break;
			case UDATokensTokenTypes.Sender:
				liste = InterpreterFactory.createAnmeldungsListe(interpreter, abschnitt, skript);
				anweisung.setSender(liste);
				break;
			case UDATokensTokenTypes.Senke:
				liste = InterpreterFactory.createAnmeldungsListe(interpreter, abschnitt, skript);
				anweisung.setSenken(liste);
				break;
			case UDATokensTokenTypes.EMPFAENGER:
				liste = InterpreterFactory.createAnmeldungsListe(interpreter, abschnitt, skript);
				anweisung.setEmpfaenger(liste);
				break;
			case UDATokensTokenTypes.Fehlerbehandlung:
				AST variable = abschnitt.getFirstChild();
				anweisung.setFehlerVariable(variable.getText());
				anweisungen = new AnweisungsBlock(InterpreterFactory.getLine(abschnitt), skript);
				AST anweisungsAst = variable.getNextSibling();
				while (anweisungsAst != null) {
					anweisungen.addAnweisung(InterpreterFactory.createAusdruck(interpreter, anweisungsAst, skript));
					anweisungsAst = anweisungsAst.getNextSibling();
				}
				anweisung.addFehlerAnweisungen(anweisungen);
				break;
			case UDATokensTokenTypes.Definition:
				anweisungen = new AnweisungsBlock(InterpreterFactory.getLine(abschnitt), skript);
				anweisungsAst = abschnitt.getFirstChild();
				while (anweisungsAst != null) {
					anweisungen.addAnweisung(InterpreterFactory.createAusdruck(interpreter, anweisungsAst, skript));
					anweisungsAst = anweisungsAst.getNextSibling();
				}
				anweisung.addDefinitionAnweisungen(anweisungen);
				break;
			case UDATokensTokenTypes.Initial:
				anweisungen = new AnweisungsBlock(InterpreterFactory.getLine(abschnitt), skript);
				anweisungsAst = abschnitt.getFirstChild();
				while (anweisungsAst != null) {
					anweisungen.addAnweisung(InterpreterFactory.createAusdruck(interpreter, anweisungsAst, skript));
					anweisungsAst = anweisungsAst.getNextSibling();
				}
				anweisung.addInitialAnweisungen(anweisungen);
				break;
			case UDATokensTokenTypes.Aktualisierung:
				variable = abschnitt.getFirstChild();
				anweisung.setAktualisierungsVariable(variable.getText());
				anweisungen = new AnweisungsBlock(InterpreterFactory.getLine(abschnitt), skript);
				anweisungsAst = variable.getNextSibling();
				while (anweisungsAst != null) {
					anweisungen.addAnweisung(InterpreterFactory.createAusdruck(interpreter, anweisungsAst, skript));
					anweisungsAst = anweisungsAst.getNextSibling();
				}
				anweisung.addAktualisierungsAnweisungen(anweisungen);
				break;
			case UDATokensTokenTypes.MENGENAENDERUNG:
				variable = abschnitt.getFirstChild();
				anweisung.setMengenAenderungsVariable(variable.getText());
				anweisungen = new AnweisungsBlock(InterpreterFactory.getLine(abschnitt), skript);
				anweisungsAst = variable.getNextSibling();
				while (anweisungsAst != null) {
					anweisungen.addAnweisung(InterpreterFactory.createAusdruck(interpreter, anweisungsAst, skript));
					anweisungsAst = anweisungsAst.getNextSibling();
				}
				anweisung.addMengenAenderungsAnweisungen(anweisungen);
				break;
			default:
				break;
			}
			abschnitt = abschnitt.getNextSibling();
		}
		return anweisung;
	}

	/**
	 * erzeugt eine Liste mit Ausdrücken für Anmeldungen innerhalb einer
	 * Anmeldungsliste einer Anmeldeanweisung.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param abschnitt
	 *            der Abschnitt der Anmeldeanweisung
	 * @param skript
	 *            der name des Skripts
	 * @return die Anmeldeliste
	 */
	private static Collection<AnmeldeAusdruck> createAnmeldungsListe(final UdaInterpreter interpreter,
			final AST abschnitt, final String skript) {
		final Collection<AnmeldeAusdruck> result = new ArrayList<>();
		AST eintrag = abschnitt.getFirstChild();
		while (eintrag != null) {
			switch (eintrag.getType()) {
			case UDATokensTokenTypes.ATGANMELDUNG:
				AST ausdruck = eintrag.getFirstChild();
				Ausdruck objekte = null;
				Ausdruck atg = null;
				Ausdruck asp = null;
				Ausdruck zeit = null;
				if (ausdruck != null) {
					objekte = InterpreterFactory.createAusdruck(interpreter, ausdruck, skript);
					ausdruck = ausdruck.getNextSibling();
				}
				if (ausdruck != null) {
					atg = InterpreterFactory.createAusdruck(interpreter, ausdruck, skript);
					ausdruck = ausdruck.getNextSibling();
				}
				if (ausdruck != null) {
					asp = InterpreterFactory.createAusdruck(interpreter, ausdruck, skript);
					ausdruck = ausdruck.getNextSibling();
				}
				if (ausdruck != null) {
					zeit = InterpreterFactory.createAusdruck(interpreter, ausdruck, skript);
					ausdruck = ausdruck.getNextSibling();
				}
				result.add(new AtgAnmeldungsEintragAusdruck(interpreter, objekte, atg, asp, zeit));
				break;
			case UDATokensTokenTypes.MENGENANMELDUNG:
				final Ausdruck mengenAusdruck = InterpreterFactory.createAusdruck(interpreter, eintrag.getFirstChild(),
						skript);
				result.add(new MengenAnmeldungsEintragAusdruck(mengenAusdruck));
				break;
			default:
				LogTools.log(InterpreterFactory.LOGGER, UdaMessage.WarningUnerwarteterAnmeldeeintrag,
						eintrag.getText());
				break;
			}
			eintrag = eintrag.getNextSibling();
		}
		return result;
	}

	/**
	 * erzeugt einen vom Uda-Interpreter auswertbaren Anweisungsausdruck.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt im vom Parser erzeugten Syntaxbaum
	 * @param skript
	 *            der Name des Skripts
	 * @return den Ausdruck
	 */
	public static Anweisung createAnweisung(final UdaInterpreter interpreter, final AST ast, final String skript) {
		Anweisung result = null;
		if (ast != null) {
			final int type = ast.getType();
			switch (type) {
			case UDATokensTokenTypes.Ausgabe:
				result = InterpreterFactory.createAusgabeAnweisung(interpreter, ast, skript, false);
				break;
			case UDATokensTokenTypes.Debug:
				result = InterpreterFactory.createAusgabeAnweisung(interpreter, ast, skript, true);
				break;
			case UDATokensTokenTypes.ZUWEISUNG:
				final AST destination = ast.getFirstChild();
				AST ausdruck = destination.getNextSibling();
				result = new ZuweisungsAnweisung(InterpreterFactory.getLine(ast), skript,
						InterpreterFactory.createAusdruck(interpreter, destination, skript),
						InterpreterFactory.createAusdruck(interpreter, ausdruck, skript));
				break;
			case UDATokensTokenTypes.Fehlerausgabe:
				ausdruck = ast.getFirstChild().getNextSibling();
				result = new MitFehlerAusgabeAnweisung(InterpreterFactory.getLine(ast), skript,
						InterpreterFactory.createAusdruck(interpreter, ausdruck, skript));
				break;
			case UDATokensTokenTypes.Fehlerpruefung:
			case UDATokensTokenTypes.Fehlerpruefung2:
				ausdruck = ast.getFirstChild().getNextSibling();
				result = new OhneFehlerPruefungAnweisung(InterpreterFactory.getLine(ast), skript,
						InterpreterFactory.createAusdruck(interpreter, ausdruck, skript));
				break;
			case UDATokensTokenTypes.Fehlerrueckgabe:
			case UDATokensTokenTypes.Fehlerrueckgabe2:
				ausdruck = ast.getFirstChild().getNextSibling();
				result = new MitFehlerRueckgabeAnweisung(InterpreterFactory.getLine(ast), skript,
						InterpreterFactory.createAusdruck(interpreter, ausdruck, skript));
				break;

			case UDATokensTokenTypes.RUECKGABEANWEISUNG:
				ausdruck = ast.getFirstChild();
				result = new RueckgabeAnweisung(InterpreterFactory.getLine(ast), skript,
						InterpreterFactory.createAusdruck(interpreter, ausdruck, skript));
				break;

			case UDATokensTokenTypes.Wenn:
				result = InterpreterFactory.createWennAnweisung(interpreter, ast, skript);
				break;

			case UDATokensTokenTypes.Zyklisch:
				result = InterpreterFactory.createZyklischeAnweisung(interpreter, ast, skript);
				break;

			case UDATokensTokenTypes.Anmeldung:
				result = InterpreterFactory.createAnmeldeAnweisung(interpreter, ast, skript);
				break;

			case UDATokensTokenTypes.Benutze:
				result = InterpreterFactory.createBenutzeAnweisung(interpreter, ast, skript);
				break;

			case UDATokensTokenTypes.Versuche:
				result = InterpreterFactory.createVersucheAnweisung(interpreter, ast, skript);
				break;

			case UDATokensTokenTypes.Ausnahme:
				result = InterpreterFactory.createAusnahmeAnweisung(interpreter, ast, skript);
				break;

			case UDATokensTokenTypes.Iteration:
				result = InterpreterFactory.createIterationsAnweisung(interpreter, ast, skript);
				break;

			default:
				break;
			}
		}
		return result;
	}

	/**
	 * erzeugt einen vom Uda-Interpreter auswertbaren Ausdruck.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhalb des vom Parser erzeugten Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return den Ausdruck
	 */
	public static Ausdruck createAusdruck(final UdaInterpreter interpreter, final AST ast, final String skript) {
		Ausdruck result = null;
		if (ast != null) {
			final int type = ast.getType();
			switch (type) {
			case UDATokensTokenTypes.Aktuell:
			case UDATokensTokenTypes.Undefiniert:
				result = new UdaBasisAusdruck(ast.getType());
				break;

			case UDATokensTokenTypes.Solange:
				final Ausdruck bedingung = InterpreterFactory.createAusdruck(interpreter, ast.getFirstChild(), skript);
				result = new BedingungsIterator(bedingung);
				break;

			case UDATokensTokenTypes.LAUFITERATOR:
				AST child = ast.getFirstChild();
				Variable variable = (Variable) InterpreterFactory.createAusdruck(interpreter, child, skript);
				child = child.getNextSibling();
				final Ausdruck von = InterpreterFactory.createAusdruck(interpreter, child, skript);
				child = child.getNextSibling();
				final Ausdruck bis = InterpreterFactory.createAusdruck(interpreter, child, skript);
				child = child.getNextSibling();
				Ausdruck schrittweite = null;
				if (child != null) {
					schrittweite = InterpreterFactory.createAusdruck(interpreter, child, skript);
				}
				result = new LaufIterationsBedingung(variable, von, bis, schrittweite);
				break;

			case UDATokensTokenTypes.MENGENITERATOR:
				child = ast.getFirstChild();
				variable = (Variable) InterpreterFactory.createAusdruck(interpreter, child, skript);
				child = child.getNextSibling();
				final Ausdruck menge = InterpreterFactory.createAusdruck(interpreter, child, skript);
				result = new MengenIterationsBedingung(variable, menge);
				break;

			case UDATokensTokenTypes.Name:
				result = new Variable(ast.getText());
				break;

			case UDATokensTokenTypes.Punkt:
				result = InterpreterFactory.erzeugeElementZugriffsOperator(interpreter, ast, skript);
				break;

			case UDATokensTokenTypes.EckigeKlammerAuf:
				result = InterpreterFactory.erzeugeContainerZugriffsOperator(interpreter, ast, skript);
				break;

			case UDATokensTokenTypes.GeschweifteKlammerAuf:
				result = InterpreterFactory.erzeugeContainerSelektor(interpreter, ast, skript);
				break;

			case UDATokensTokenTypes.QUANTOR:
				result = InterpreterFactory.createQuantorAusdruck(interpreter, ast.getFirstChild(), skript);
				break;

			case UDATokensTokenTypes.Quantor:
				final AST quantorDaten = ast.getFirstChild();
				if (quantorDaten == null) {
					throw new InterpreterException(
							"Zeile: " + ast.getLine() + ": Die Quantordefinition ist nicht vollständig");
				}
				result = InterpreterFactory.createQuantorDefinition(interpreter, ast.getFirstChild(), skript);
				break;

			case UDATokensTokenTypes.FUNKTIONSAUFRUF:
				result = InterpreterFactory.createFunktionsAufruf(interpreter, ast.getFirstChild(), skript);
				break;

			case UDATokensTokenTypes.Funktion:
				result = InterpreterFactory.createFunktionsDefinition(interpreter, ast.getFirstChild(), skript);
				break;

			case UDATokensTokenTypes.ALLE:
			case UDATokensTokenTypes.EIN:
			case UDATokensTokenTypes.Mindestens:
			case UDATokensTokenTypes.Hoechstens:
			case UDATokensTokenTypes.Hoechstens2:
				result = InterpreterFactory.createQuantorBedingung(interpreter, ast, skript);
				break;

			case UDATokensTokenTypes.Zahl:
				final String text = ast.getText();
				if (text.toUpperCase().contains("E") || text.contains(".")) {
					result = new FliessKommaZahlLiteral(Double.parseDouble(text));
				} else {
					result = new GanzZahlLiteral(Long.parseLong(text));
				}
				break;
			case UDATokensTokenTypes.Zeichenkette:
				result = new ZeichenkettenLiteral(ast.getText());
				break;
			case UDATokensTokenTypes.Wahr:
				result = new BooleanLiteral(true);
				break;
			case UDATokensTokenTypes.Falsch:
				result = new BooleanLiteral(false);
				break;

			case UDATokensTokenTypes.ZEIT:
				result = new RelativeZeitLiteral();
				AST value = null;
				AST unit = null;
				for (int idx = 0; idx < (ast.getNumberOfChildren() / 2); idx++) {
					if (unit == null) {
						value = ast.getFirstChild();
					} else {
						value = unit.getNextSibling();
					}
					if (value != null) {
						unit = value.getNextSibling();
					}
					if ((value != null) && (unit != null)) {
						RelativeZeitLiteral.ZeitEinheit einheit = RelativeZeitLiteral.ZeitEinheit.MILLISEKUNDE;
						switch (unit.getType()) {
						case UDATokensTokenTypes.Millisekunden:
						case UDATokensTokenTypes.Millisekunde:
							einheit = RelativeZeitLiteral.ZeitEinheit.MILLISEKUNDE;
							break;
						case UDATokensTokenTypes.Sekunden:
						case UDATokensTokenTypes.Sekunde:
							einheit = RelativeZeitLiteral.ZeitEinheit.SEKUNDE;
							break;
						case UDATokensTokenTypes.Minuten:
						case UDATokensTokenTypes.Minute:
							einheit = RelativeZeitLiteral.ZeitEinheit.MINUTE;
							break;
						case UDATokensTokenTypes.Stunden:
						case UDATokensTokenTypes.Stunde:
							einheit = RelativeZeitLiteral.ZeitEinheit.STUNDE;
							break;
						case UDATokensTokenTypes.Tage:
						case UDATokensTokenTypes.Tag:
							einheit = RelativeZeitLiteral.ZeitEinheit.TAG;
							break;
						default:
							break;
						}
						final Ausdruck wertAusdruck = InterpreterFactory.createAusdruck(interpreter, value, skript);
						if (wertAusdruck instanceof ZahlenLiteral) {
							((RelativeZeitLiteral) result).addZeitAngabe((ZahlenLiteral) wertAusdruck, einheit);
						} else {
							throw new InterpreterException("Der Wert für die Zeitangabe ist keine Zahl");
						}
					}
				}
				break;

			case UDATokensTokenTypes.FELD:
				final Collection<Ausdruck> elemente = new ArrayList<>();
				AST feldElement = ast.getFirstChild();
				while (feldElement != null) {
					elemente.add(InterpreterFactory.createAusdruck(interpreter, feldElement, skript));
					feldElement = feldElement.getNextSibling();
				}
				result = new FeldInitialisiererSymbol(elemente);
				break;

			case UDATokensTokenTypes.LISTE:
				final Collection<Ausdruck> listenElemente = new ArrayList<>();
				AST listenElement = ast.getFirstChild();
				while (listenElement != null) {
					listenElemente.add(InterpreterFactory.createAusdruck(interpreter, listenElement, skript));
					listenElement = listenElement.getNextSibling();
				}
				result = new ListenInitialisiererSymbol(listenElemente);
				break;

			case UDATokensTokenTypes.SELEKTOR:
				result = InterpreterFactory.createSelector(interpreter, ast.getFirstChild(), skript);
				break;

			case UDATokensTokenTypes.Minus:
				AST links = ast.getFirstChild();
				AST rechts = links.getNextSibling();
				if (rechts != null) {
					result = new Operation(ast.getText(),
							InterpreterFactory.createAusdruck(interpreter, links, skript),
							InterpreterFactory.createAusdruck(interpreter, rechts, skript));
				} else {
					result = new Operation(ast.getText(),
							InterpreterFactory.createAusdruck(interpreter, links, skript));
				}
				break;

			case UDATokensTokenTypes.Plus:
			case UDATokensTokenTypes.Mal:
			case UDATokensTokenTypes.Dividiert:
			case UDATokensTokenTypes.Und:
			case UDATokensTokenTypes.Oder:
			case UDATokensTokenTypes.Ampersand:
			case UDATokensTokenTypes.Div:
			case UDATokensTokenTypes.Modulo:
			case UDATokensTokenTypes.Ungleich:
			case UDATokensTokenTypes.Kleiner:
			case UDATokensTokenTypes.Groesser:
			case UDATokensTokenTypes.Gleich:
			case UDATokensTokenTypes.KleinerGleich:
			case UDATokensTokenTypes.GroesserGleich:
				links = ast.getFirstChild();
				rechts = links.getNextSibling();
				result = new Operation(ast.getText(),
						InterpreterFactory.createAusdruck(interpreter, links, skript),
						InterpreterFactory.createAusdruck(interpreter, rechts, skript));
				break;

			case UDATokensTokenTypes.Nicht:
				links = ast.getFirstChild();
				result = new Operation(ast.getText(),
						InterpreterFactory.createAusdruck(interpreter, links, skript));
				break;

			case UDATokensTokenTypes.KlammerAuf:
				break;

			case UDATokensTokenTypes.EOF:
			case UDATokensTokenTypes.NULL_TREE_LOOKAHEAD:
			case UDATokensTokenTypes.ZeilenEnde:
			case UDATokensTokenTypes.Mit:
			case UDATokensTokenTypes.Ohne:
			case UDATokensTokenTypes.Der:
			case UDATokensTokenTypes.Elemente:
			case UDATokensTokenTypes.In:
			case UDATokensTokenTypes.Prozent:
			case UDATokensTokenTypes.Die:
			case UDATokensTokenTypes.Bedingung:
			case UDATokensTokenTypes.Ist:
			case UDATokensTokenTypes.Gilt:
			case UDATokensTokenTypes.Doppelpunkt:
			case UDATokensTokenTypes.Komma:
			case UDATokensTokenTypes.EckigeKlammerZu:
			case UDATokensTokenTypes.GeschweifteKlammerZu:
			case UDATokensTokenTypes.KlammerZu:
			case UDATokensTokenTypes.Ende:
			case UDATokensTokenTypes.Von:
			case UDATokensTokenTypes.Bis:
			case UDATokensTokenTypes.Schrittweite:
			case UDATokensTokenTypes.Alle:
			case UDATokensTokenTypes.Dann:
			case UDATokensTokenTypes.Sonst:
			case UDATokensTokenTypes.Quelle:
			case UDATokensTokenTypes.Sender:
			case UDATokensTokenTypes.Senke:
			case UDATokensTokenTypes.Fehlerbehandlung:
			case UDATokensTokenTypes.Initial:
			case UDATokensTokenTypes.Jeweils:
			case UDATokensTokenTypes.Bei:
			case UDATokensTokenTypes.Aktualisierung:
			case UDATokensTokenTypes.Timer:
			case UDATokensTokenTypes.Als:
			case UDATokensTokenTypes.Menge:
			case UDATokensTokenTypes.Millisekunden:
			case UDATokensTokenTypes.Sekunden:
			case UDATokensTokenTypes.Minuten:
			case UDATokensTokenTypes.Stunden:
			case UDATokensTokenTypes.Tage:
			case UDATokensTokenTypes.Millisekunde:
			case UDATokensTokenTypes.Sekunde:
			case UDATokensTokenTypes.Minute:
			case UDATokensTokenTypes.Stunde:
			case UDATokensTokenTypes.Tag:
			case UDATokensTokenTypes.Jede:
			case UDATokensTokenTypes.Jedes:
			case UDATokensTokenTypes.Jeden:
			case UDATokensTokenTypes.Dem:
			case UDATokensTokenTypes.Den:
			case UDATokensTokenTypes.Ein:
			case UDATokensTokenTypes.Einen:
			case UDATokensTokenTypes.Eine:
			case UDATokensTokenTypes.Empfaenger:
			case UDATokensTokenTypes.Empfaenger2:
			case UDATokensTokenTypes.Erfuellt:
			case UDATokensTokenTypes.Erfuellt2:
			case UDATokensTokenTypes.MengenAenderung:
			case UDATokensTokenTypes.MengenAenderung2:
			case UDATokensTokenTypes.Ueber:
			case UDATokensTokenTypes.Ueber2:
			default:
				break;
			}
		}

		if (result == null) {
			result = InterpreterFactory.createAnweisung(interpreter, ast, skript);
		}

		return result;
	}

	/**
	 * erzeugt einen Ausdruck für eine Ausgabeanweisung.
	 *
	 * @param interpreter
	 *            det verwendete Interpreter
	 * @param ast
	 *            der Startpunkt im vom Parser gelieferten Syntaxbaum
	 * @param skript
	 *            der Name des Skripts
	 * @param debug
	 *            Ausgabe erfolgt im Debug-Format (mit Zeilennummer) ?
	 * @return die Anweisung
	 */
	private static Anweisung createAusgabeAnweisung(final UdaInterpreter interpreter, final AST ast,
			final String skript, final boolean debug) {
		final AST child = ast.getFirstChild();
		AST ausdrucksAst = null;
		ausdrucksAst = child;
		return new AusgabeAnweisung(InterpreterFactory.getLine(ast), skript, debug,
				InterpreterFactory.createAusdruck(interpreter, ausdrucksAst, skript));
	}

	/**
	 * erzeugt einen Ausdruck für eine Ausnahmeanweisung.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt im Syntaxbaum
	 * @param skript
	 *            der Name des Skripts
	 * @return die Anweisung
	 */
	private static Anweisung createAusnahmeAnweisung(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		final AST ausdruckAST = ast.getFirstChild();
		return new AusnahmeAnweisung(InterpreterFactory.getLine(ast), skript,
				InterpreterFactory.createAusdruck(interpreter, ausdruckAST, skript));
	}

	/**
	 * erzeugt den Ausdruck für eine Benutze-Anweisung.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt im vom Parser gelieferten Syntaxbaum
	 * @param skript
	 *            der Name des Skripts
	 * @return die Anweisung
	 */
	private static Anweisung createBenutzeAnweisung(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		final AST nameAst = ast.getFirstChild();
		return new BenutzeAnweisung(InterpreterFactory.getLine(ast), skript,
				(ZeichenkettenLiteral) InterpreterFactory.createAusdruck(interpreter, nameAst, skript));
	}

	/**
	 * erzeugt einen Ausdruck für einen Funktiopnsaufruf.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhalb des vom Parser gelieferten
	 *            Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return den Ausdruck
	 */
	private static Ausdruck createFunktionsAufruf(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		final String name = ast.getText();
		final List<Ausdruck> argumente = new ArrayList<>();

		AST argAst = ast.getNextSibling();
		while (argAst != null) {
			argumente.add(InterpreterFactory.createAusdruck(interpreter, argAst, skript));
			argAst = argAst.getNextSibling();
		}

		return new FunktionsAufruf(name, argumente);
	}

	/**
	 * erzeugt den Ausdruck für eine Funktionsdefinition.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhlab des vom Parser gelieferten
	 *            Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return die Definition
	 */
	private static Ausdruck createFunktionsDefinition(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		final String name = ast.getText();
		final List<String> argumente = new ArrayList<>();
		final AnweisungsBlock anweisungen = new AnweisungsBlock(InterpreterFactory.getLine(ast), skript);

		AST argAst = ast.getFirstChild();
		while (argAst != null) {
			argumente.add(argAst.getText());
			argAst = argAst.getNextSibling();
		}

		AST anweisungsAst = ast.getNextSibling();
		while (anweisungsAst != null) {
			anweisungen.addAnweisung(InterpreterFactory.createAusdruck(interpreter, anweisungsAst, skript));
			anweisungsAst = anweisungsAst.getNextSibling();
		}

		return new FunktionsDefinition(InterpreterFactory.getLine(ast), skript, name, argumente, anweisungen);
	}

	/**
	 * erzeugt den Interpreterbaum aus dem Quelltext des Skripts.<br>
	 * Ein Parser erzeugt einen AST, der in den den Interpreterbaum umgesetzt
	 * wird. Der Interpreterbaum setzt sich aus Ausdrücken zusammen, die vom
	 * Interpreter ausgewertet, d.h. ausgeführt werden können.
	 *
	 * @param initial
	 *            Kennzeichnung, ob das ursprüngliche oder ein eingebundenes
	 *            Skript übersetzt wird.
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param skript
	 *            der Name des Skripts
	 * @throws UdaInterpreterException
	 *             der Quelltext des Skripts konnte nicht in einen gültigen
	 *             Interpreterbaum übersetzt werden
	 */
	public static synchronized void createInterpreterBaum(final boolean initial, final UdaInterpreter interpreter,
			final Skript skript) throws UdaInterpreterException {

		if (initial) {
			InterpreterFactory.ANALYSIERT.clear();
		}

		if (InterpreterFactory.ANALYSIERT.contains(skript)) {
			return;
		}

		InterpreterFactory.ANALYSIERT.add(skript);

		final UDALexer lexer = new UDALexer(new StringReader(skript.getSource()));

		final InterpreterFactory parser = new InterpreterFactory(lexer);
		parser.setFilename(skript.getName());
		parser.setASTNodeClass(CommonASTWithLines.class.getName());

		try {
			parser.skript();

			AST ast = parser.getAST();

			while (ast != null) {
				final Ausdruck ausdruck = InterpreterFactory.createAusdruck(interpreter, ast, skript.getName());
				if (ausdruck instanceof FunktionsDefinition) {
					final FunktionsDefinition definition = (FunktionsDefinition) ausdruck;
					interpreter.setFunktion(definition.getName(), definition);
				} else if (ausdruck instanceof QuantorDefinition) {
					final QuantorDefinition definition = (QuantorDefinition) ausdruck;
					interpreter.setQuantor(definition.getName(), definition);
				} else if (ausdruck instanceof BenutzeAnweisung) {
					final BenutzeAnweisung benutze = (BenutzeAnweisung) ausdruck;
					try {
						final String name = benutze.interpret(null);
						Skript benutztesSkript = null;
						if (skript instanceof LokalesSkript) {
							benutztesSkript = new LokalesSkript(name);
						} else if (skript instanceof ServerSkript) {
							benutztesSkript = ((ServerSkript) skript).getModul().getSkriptWithName(name);
						} else if (skript instanceof ClientSkript) {
							benutztesSkript = ((ClientSkript) skript).getModul().getSkriptWithName(name);
						}
						InterpreterFactory.createInterpreterBaum(false, interpreter, benutztesSkript);
					} catch (final UdaInterpreterException e) {
						parser.fehlerListe.addAll(e.getFehlerListe());
					}
				} else if (ausdruck instanceof Anweisung) {
					interpreter.addAnweisung(ausdruck);
				}

				ast = ast.getNextSibling();
			}
		} catch (final RecognitionException e) {
			parser.fehlerListe.add(new UdaParserFehler(e));
		} catch (final TokenStreamException e) {
			parser.fehlerListe.add(new UdaParserFehler(e));
		}

		if (parser.fehlerListe.size() > 0) {
			throw new UdaInterpreterException(parser.fehlerListe);
		}

	}

	/**
	 * erzeugt den Ausdruck für eine Iterationsanweisung.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhalb des vom Parser gelieferten
	 *            Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return die Anweisung
	 */
	private static Anweisung createIterationsAnweisung(final UdaInterpreter interpreter, final AST ast,
			final String skript) {

		AST part = ast.getFirstChild();
		final IterationsBedingung iteration = (IterationsBedingung) InterpreterFactory.createAusdruck(interpreter, part,
				skript);
		final AnweisungsBlock anweisungen = new AnweisungsBlock(InterpreterFactory.getLine(ast), skript);

		part = part.getNextSibling();
		while (part != null) {
			anweisungen.addAnweisung(InterpreterFactory.createAusdruck(interpreter, part, skript));
			part = part.getNextSibling();
		}
		return new IterationsAnweisung(InterpreterFactory.getLine(ast), skript, iteration, anweisungen);
	}

	/**
	 * erzeugt den Ausdruck für einen Quantor.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhalb des vom Parser gelieferten
	 *            Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return den Ausdruck
	 */
	private static Ausdruck createQuantorAusdruck(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		final QuantorBedingung quantitaet = (QuantorBedingung) InterpreterFactory.createQuantorBedingung(interpreter,
				ast, skript);
		AST part = ast.getNextSibling();
		final Variable variable = (Variable) InterpreterFactory.createAusdruck(interpreter, part, skript);
		part = part.getNextSibling();
		final Ausdruck menge = InterpreterFactory.createAusdruck(interpreter, part, skript);
		part = part.getNextSibling();
		final Ausdruck bedingung = InterpreterFactory.createAusdruck(interpreter, part, skript);
		return new Quantor(variable, quantitaet, menge, bedingung);
	}

	/**
	 * erzeugt den Ausdruck für eine Quantorbedingung.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhalb des vom Parser gelieferten
	 *            Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return den Ausdruck
	 */
	private static Ausdruck createQuantorBedingung(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		int typ = QuantorBedingung.UNDEFINED;
		boolean prozent = false;
		Ausdruck ausdruck = null;
		final String name = ast.getText();
		switch (ast.getType()) {
		case UDATokensTokenTypes.ALLE:
			typ = QuantorBedingung.ALLE;
			break;
		case UDATokensTokenTypes.EIN:
			typ = QuantorBedingung.EIN;
			break;
		case UDATokensTokenTypes.Hoechstens:
		case UDATokensTokenTypes.Hoechstens2:
			typ = QuantorBedingung.HOECHSTENS;
			break;
		case UDATokensTokenTypes.Mindestens:
			typ = QuantorBedingung.MINDESTENS;
			break;
		case UDATokensTokenTypes.Name:
			typ = QuantorBedingung.NUTZER;
			break;
		default:
			break;
		}

		AST child = ast.getFirstChild();
		if (child != null) {
			if (child.getType() == UDATokensTokenTypes.Prozent) {
				prozent = true;
				child = child.getFirstChild();
				if (child != null) {
					ausdruck = InterpreterFactory.createAusdruck(interpreter, child, skript);
				}
			} else {
				ausdruck = InterpreterFactory.createAusdruck(interpreter, child, skript);
			}
		}

		return new QuantorBedingung(typ, name, ausdruck, prozent);
	}

	/**
	 * erzeugt einen Ausdruck für eine Quantordefinition.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhalb des vom Parser gelieferten
	 *            Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return die Definition
	 */
	private static Ausdruck createQuantorDefinition(final UdaInterpreter interpreter, final AST ast,
			final String skript) {

		final String name = ast.getText();
		final AST child = ast.getFirstChild();
		final String mengenName = child.getText();
		final AST bedingungsKnoten = child.getNextSibling();
		final String bedingungsName = bedingungsKnoten.getText();
		final AnweisungsBlock anweisungen = new AnweisungsBlock(InterpreterFactory.getLine(ast), skript);
		AST part = ast.getNextSibling();

		while (part != null) {
			anweisungen.addAnweisung(InterpreterFactory.createAusdruck(interpreter, part, skript));
			part = part.getNextSibling();
		}

		return new QuantorDefinition(name, mengenName, bedingungsName, anweisungen);
	}

	/**
	 * erzeugt einen Selektor-Ausdruck.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhalb des vom Parser gelieferten
	 *            Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return den Ausdruck
	 */
	private static Ausdruck createSelector(final UdaInterpreter interpreter, final AST ast, final String skript) {
		final ContainerSelektorSymbol selektor = new ContainerSelektorSymbol();
		AST nameAst = ast;
		while (nameAst != null) {
			final AST valueAst = nameAst.getNextSibling();
			if (valueAst != null) {
				final String name = nameAst.getText();
				final Ausdruck value = InterpreterFactory.createAusdruck(interpreter, valueAst, skript);
				selektor.addBedingung(name, value);
				nameAst = valueAst.getNextSibling();
			} else {
				nameAst = null;
			}
		}

		return selektor;
	}

	/**
	 * erzeugt einen Ausdruck für eine Versucheanweisung.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhalb des vom Parser gelieferten
	 *            Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return die Anweisung
	 */
	private static Anweisung createVersucheAnweisung(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		final AnweisungsBlock anweisungen = new AnweisungsBlock(InterpreterFactory.getLine(ast), skript);
		Variable fehlerVariable = null;
		AnweisungsBlock fehlerBehandlung = null;

		AST currentAst = ast.getFirstChild();
		while (currentAst != null) {
			if (currentAst.getType() == UDATokensTokenTypes.Fehlerbehandlung) {
				fehlerBehandlung = new AnweisungsBlock(InterpreterFactory.getLine(currentAst), skript);
				currentAst = currentAst.getFirstChild();
				fehlerVariable = (Variable) InterpreterFactory.createAusdruck(interpreter, currentAst, skript);
			} else {
				if (fehlerBehandlung != null) {
					fehlerBehandlung.addAnweisung(InterpreterFactory.createAusdruck(interpreter, currentAst, skript));
				} else {
					anweisungen.addAnweisung(InterpreterFactory.createAusdruck(interpreter, currentAst, skript));
				}
			}
			currentAst = currentAst.getNextSibling();
		}

		return new VersucheAnweisung(InterpreterFactory.getLine(ast), skript, anweisungen, fehlerVariable,
				fehlerBehandlung);
	}

	/**
	 * erzeugt den Ausdruck für eine WennAnweisung.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhalb des vom Parser gelieferten
	 *            Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return die Anweisung
	 */
	private static Anweisung createWennAnweisung(final UdaInterpreter interpreter, final AST ast, final String skript) {

		AST part = ast.getFirstChild();

		Ausdruck bedingung = InterpreterFactory.createAusdruck(interpreter, part, skript);
		final Collection<Anweisung> liste = new ArrayList<>();
		final List<BedingterAnweisungsBlock> alternativen = new ArrayList<>();
		do {
			if (part != null) {
				part = part.getNextSibling();
			}
			if (part != null) {
				AST anweisung = part.getFirstChild();
				switch (part.getType()) {
				case UDATokensTokenTypes.Dann:
					liste.clear();
					while (anweisung != null) {
						liste.add((Anweisung) InterpreterFactory.createAusdruck(interpreter, anweisung, skript));
						anweisung = anweisung.getNextSibling();
					}
					alternativen.add(
							new BedingterAnweisungsBlock(InterpreterFactory.getLine(part), skript, bedingung, liste));
					break;

				case UDATokensTokenTypes.Wenn:
					liste.clear();
					bedingung = InterpreterFactory.createAusdruck(interpreter, anweisung, skript);
					anweisung = anweisung.getNextSibling().getFirstChild();
					while (anweisung != null) {
						liste.add((Anweisung) InterpreterFactory.createAusdruck(interpreter, anweisung, skript));
						anweisung = anweisung.getNextSibling();
					}
					alternativen.add(
							new BedingterAnweisungsBlock(InterpreterFactory.getLine(part), skript, bedingung, liste));
					break;

				case UDATokensTokenTypes.Sonst:
					bedingung = new BooleanLiteral(true);
					liste.clear();
					while (anweisung != null) {
						liste.add((Anweisung) InterpreterFactory.createAusdruck(interpreter, anweisung, skript));
						anweisung = anweisung.getNextSibling();
					}
					alternativen.add(
							new BedingterAnweisungsBlock(InterpreterFactory.getLine(part), skript, bedingung, liste));
					break;
				default:
					break;
				}
			}
		} while (part != null);
		return new WennAnweisung(InterpreterFactory.getLine(ast), skript, alternativen);
	}

	/**
	 * erzeugt den Ausdruck für eine zyklische Anweisung.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            der Startpunkt innerhalb des vom Parser gelieferten
	 *            Syntaxbaums
	 * @param skript
	 *            der Name des Skripts
	 * @return die Anweisung
	 */
	private static Anweisung createZyklischeAnweisung(final UdaInterpreter interpreter, final AST ast,
			final String skript) {

		Variable variable = null;
		RelativeZeitLiteral zyklus = null;
		final AnweisungsBlock anweisungen = new AnweisungsBlock(InterpreterFactory.getLine(ast), skript);

		AST part = ast.getFirstChild();
		while (part != null) {
			final Ausdruck ausdruck = InterpreterFactory.createAusdruck(interpreter, part, skript);
			if (ausdruck instanceof Variable) {
				variable = (Variable) ausdruck;
			} else if (ausdruck instanceof RelativeZeitLiteral) {
				zyklus = (RelativeZeitLiteral) ausdruck;
			} else if (ausdruck instanceof Anweisung) {
				anweisungen.addAnweisung(ausdruck);
			}
			part = part.getNextSibling();
		}
		return new ZyklischeAnweisung(InterpreterFactory.getLine(ast), skript, variable, zyklus, anweisungen);
	}

	/**
	 * erzeugt einen Ausdruck für die Selektion innerhalb eines Containers.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            das Element aus dem Syntaxbaum
	 * @param skript
	 *            das Skript, in dem der Ausdruck enthalten ist
	 * @return der erzeugte Ausdruck
	 */
	private static Ausdruck erzeugeContainerSelektor(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		final Ausdruck result;
		final AST links = ast.getFirstChild();
		final AST rechts = links.getNextSibling();
		result = new ContainerMehrfachSelektionsSymbol(InterpreterFactory.createAusdruck(interpreter, links, skript),
				InterpreterFactory.createAusdruck(interpreter, rechts, skript));
		return result;
	}

	/**
	 * erzeugt einen Ausdruck für den Zugriff auf ein Containerelement.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            das Element aus dem Syntaxbaum
	 * @param skript
	 *            das Skript, in dem der Ausdruck enthalten ist
	 * @return der erzeugte Ausdruck
	 */
	private static Ausdruck erzeugeContainerZugriffsOperator(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		final Ausdruck result;
		final AST links = ast.getFirstChild();
		final AST rechts = links.getNextSibling();
		result = new ContainerZugriffsSymbol(InterpreterFactory.createAusdruck(interpreter, links, skript),
				InterpreterFactory.createAusdruck(interpreter, rechts, skript));
		return result;
	}

	/**
	 * erzeugt einen Ausdruck für den Elementzugrifssoperator.
	 *
	 * @param interpreter
	 *            der verwendete Interpreter
	 * @param ast
	 *            das Element aus dem Syntaxbaum
	 * @param skript
	 *            das Skript, in dem der Ausdruck enthalten ist
	 * @return der erzeugte Ausdruck
	 */
	private static Ausdruck erzeugeElementZugriffsOperator(final UdaInterpreter interpreter, final AST ast,
			final String skript) {
		final Ausdruck result;
		final AST links = ast.getFirstChild();
		final AST rechts = links.getNextSibling();
		result = new ElementZugriffsSymbol(InterpreterFactory.createAusdruck(interpreter, links, skript),
				InterpreterFactory.createAusdruck(interpreter, rechts, skript));
		return result;
	}

	/**
	 * sucht rekursiv für den Knoten eine gültige Zeilennummer.<br>
	 * Workaround, weil mit Strings übersetzte Tokens nicht mit der Zeilennummer
	 * initialisiert werden.
	 *
	 * @param ast
	 *            der Knoten
	 * @return die Zeilennummer
	 */
	private static int getLine(final AST ast) {
		int result = ast.getLine();

		AST child = ast.getFirstChild();
		while (child != null) {
			if (result > 0) {
				break;
			}
			result = InterpreterFactory.getLine(child);
			child = child.getNextSibling();
		}
		return result;
	}

	/**
	 * die Liste der bei der Umsetzung aufgelaufenen Fehler.
	 */
	private final Collection<UdaParserFehler> fehlerListe = new ArrayList<>();

	/**
	 * erzeugt eine Instanz der Klasse unter Verwendung des übergebenen Lexers.
	 *
	 * @param lexer
	 *            der Lexer
	 */
	private InterpreterFactory(final UDALexer lexer) {
		super(lexer);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see antlr.Parser#reportError(antlr.RecognitionException)
	 */
	@Override
	public void reportError(final RecognitionException ex) {
		fehlerListe.add(new UdaParserFehler(ex));
	}
}
