/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * die Repräsentation einer Anweisung zum Werfen einer Ausnahme.<br>
 * Die Funktion wertet den enthaltenen Ausdruck aus, um ein Fehlerobjekt zu
 * erzeugen. Das Fehlerobjekt wird als Java-Runtimeexception geworfen und muß
 * innerhalb des Uda-Skripts über eine Methode der Fehlerbehandlung abgefangen
 * werden:
 * <ul>
 * <li>Versuche-Anweisung</li>
 * <li>Fehlerbehandlungsblock in einer Anmeldeanweisung</li>
 * </ul>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class AusnahmeAnweisung extends AbstractEinfacheAnweisung {

	/**
	 * Konstruktor.
	 *
	 * @param zeile
	 *            die Nummer der Zeile, in der die Anweisung im Quelltext
	 *            definiert ist
	 * @param skriptname
	 *            der Name des Skripts
	 * @param ausdruck
	 *            der Ausdruck, mit dem das Fehlerobjekt erzeugt wird.
	 */
	public AusnahmeAnweisung(final int zeile, final String skriptname, final Ausdruck ausdruck) {
		super(zeile, skriptname, ausdruck);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion wertet den Ausdruck der Anweisung aus, um ein Fehlerobjekt
	 * zu erzeugen. Ist das Ergebnis selbst kein Fehlerobjekt wird aus diesem
	 * Fehler selbst ein Fehlerobjekt erzeugt.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		try {
			Object fehler = getAusdruck().interpret(kontext);
			if (!(fehler instanceof UdaFehler)) {
				fehler = new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
			}
			throw (UdaFehler) fehler;
		} catch (final InterpreterException e) {
			throw erzeugeAnweisungsFehler(e);
		}
	}
}
