/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung;

import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * die Operatoren für den Handler zur Ausführung von Operationen mit einer
 * Anmeldungs-Aktivität.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public final class AnmeldungOperatoren {

	/**
	 * Ermitteln der Senke-Anmeldungen einer Anmeldungs-Aktivität.
	 */
	public static final Operator SENKE_ANMELDUNGEN = Operator.getOperator("senkeAnmeldungen");

	/**
	 * Ermitteln der Empfänger-Anmeldungen einer Anmeldungs-Aktivität.
	 */
	public static final Operator EMPFAENGER_ANMELDUNGEN = Operator.getOperator("empfängerAnmeldungen");

	/**
	 * Ermitteln der Sender-Anmeldungen einer Anmeldungs-Aktivität.
	 */
	public static final Operator SENDER_ANMELDUNGEN = Operator.getOperator("senderAnmeldungen");

	/**
	 * Ermitteln der Quelle-Anmeldungen einer Anmeldungs-Aktivität.
	 */
	public static final Operator QUELLE_ANMELDUNGEN = Operator.getOperator("quelleAnmeldungen");

	/**
	 * Hinzufügen einer Empfänger- bzw. Senkeanmeldung.
	 */
	public static final Operator LESEN = Operator.getOperator("lesen");

	/**
	 * Entfernen einer Empfänger- bzw. Senkeanmeldung.
	 */
	public static final Operator NICHT_LESEN = Operator.getOperator("nichtLesen");

	/**
	 * Hinzufügen einer Sender- bzw. Quelleanmeldung.
	 */
	public static final Operator SCHREIBEN = Operator.getOperator("schreiben");

	/**
	 * Entfernen einer Sender- bzw. Quelleanmeldung.
	 */
	public static final Operator NICHT_SCHREIBEN = Operator.getOperator("nichtSchreiben");

	/**
	 * die Liste aller unterstützter Operatoren.
	 */
	public static final Operator[] OPERATOREN = { AnmeldungOperatoren.SENKE_ANMELDUNGEN,
			AnmeldungOperatoren.EMPFAENGER_ANMELDUNGEN, AnmeldungOperatoren.SENDER_ANMELDUNGEN,
			AnmeldungOperatoren.QUELLE_ANMELDUNGEN, AnmeldungOperatoren.LESEN, AnmeldungOperatoren.NICHT_LESEN,
			AnmeldungOperatoren.SCHREIBEN, AnmeldungOperatoren.NICHT_SCHREIBEN

	};

	/** privater Konstruktor für Utility-Klasse. */
	private AnmeldungOperatoren() {
		super();
	}

}
