/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.dav;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.uda.interpreter.daten.container.Feld;

/**
 * die Repräsentation eines Datenverteiler-<i>AttributeGroup</i>-Objekts in
 * einem Uda-Skript.<br>
 * Das Objekt hat, bis auf die vom Typ <i>KonfigurationsObjekt</i> vererbten:
 * <ul>
 * <li>pid</li>
 * <li>id</li>
 * <li>name</li>
 * <li>info</li>
 * <li>typ</li>
 * <li>gültig</li>
 * <li>attributgruppen</li>
 * <li>mengen</li>
 * </ul>
 * folgende zusätzliche Attribute:
 * <ul>
 * <li><b>aspekte:</b> liefert die Liste der Aspekte, die für die Attributgruppe
 * definiert sind.</li>
 * <li><b>attribute:</b> liefert die Liste der Attribute der Attributgruppe</li>
 * </ul>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class AtgObjekt extends KonfigurationsObjekt {

	/**
	 * Konstruktor.
	 *
	 * @param verbindung
	 *            die verwendete Datenverteilerverbindung
	 * @param object
	 *            das <i>AttributeGroup</i>-Objekt des Datenverteilers
	 */
	public AtgObjekt(final ClientDavInterface verbindung, final AttributeGroup object) {
		super(verbindung, object);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Es stehen folgende Attribute zur Verfügung:
	 * <ul>
	 * <li>pid: die PID der Attributgruppe</li>
	 * <li>id: die ID der Attributgruppe</li>
	 * <li>name: der Name der Attributgruppe</li>
	 * <li>info: das Info-Objekt der Attributgruppe (enthält <i>kurzinfo</i> und
	 * <i>beschreibung</i></li>
	 * <li>typ: der Typ der Attributgruppe</li>
	 * <li>gültig: die Gültigkeit der Attributgruppe zum aktuellen Zeitpunkt
	 * </li>
	 * <li>attributgruppen: die Liste der Attributgruppen des Typs der
	 * Attributgruppe</li>
	 * <li>mengen: die Liste der Mengen des Types der Attributgruppe</li>
	 * <li><b>aspekte:</b> die Liste der Aspekte, die für die Attributgruppe
	 * definiert sind.</li>
	 * <li><b>attribute:</b> die Liste der Attribute der Attributgruppe</li>
	 * </ul>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.dav.KonfigurationsObjekt#getStrukturElement(java.lang.String)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object result = null;

		final AttributeGroup atg = getObjekt();

		if ("aspekte".equals(name)) {
			result = new Feld();
			for (final SystemObject obj : atg.getAspects()) {
				((Feld) result).einfuegen(0, DavDaten.erzeugeObjekt(getVerbindung(), obj));
			}
		} else if ("attribute".equals(name)) {
			result = new Feld();
			for (final Object obj : atg.getAttributes()) {
				((Feld) result).einfuegen(0, DavDaten.erzeugeObjekt(getVerbindung(), obj));
			}
		}

		if (result == null) {
			result = super.getStrukturElement(name);
		}

		return result;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion liefert immer ein <i>AttributeGroup</i>-Objekt im Gegensatz
	 * zur allgemeineren Definition in <i>DavDaten</i>.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.dav.DavDaten#getObjekt()
	 */
	@Override
	public AttributeGroup getObjekt() {
		return (AttributeGroup) super.getObjekt();
	}
}
