/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.dav;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.ibv.uda.interpreter.daten.container.Feld;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * die Repräsentation eines Ergebnisdatensatzes ({@link ResultData}) vom
 * Datenverteiler innerhalb des Uda-Interpreters.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class DavDatenSatz extends DavDaten {

	/**
	 * Konstruktor.<br>
	 * Erzeugt eine Instanz der Klasse, die den über die übergebene
	 * Datenverteilerverbindung Datensatz kapselt.
	 *
	 * @param verbindung
	 *            die verwendete Datenverteilerverbindung
	 * @param objekt
	 *            das zu Grunde liegende ResultData-Objekt
	 */
	protected DavDatenSatz(final ClientDavInterface verbindung, final ResultData objekt) {
		super(verbindung, objekt);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.dav.DavDaten#getObjekt()
	 */
	@Override
	public ResultData getObjekt() {
		return (ResultData) super.getObjekt();
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#getStrukturElement(java.lang.String)
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object result = null;
		final ResultData object = getObjekt();

		if ("attributgruppe".equals(name)) {
			result = DavDaten.erzeugeObjekt(getVerbindung(), object.getDataDescription().getAttributeGroup());
		} else if ("aspekt".equals(name)) {
			result = DavDaten.erzeugeObjekt(getVerbindung(), object.getDataDescription().getAspect());
		} else if ("attribute".equals(name)) {
			result = new Feld();
			final Data datenSatz = object.getData();
			for (final Attribute attribut : object.getDataDescription().getAttributeGroup().getAttributes()) {
				if (datenSatz != null) {
					((Feld) result).einfuegen(0,
							DavDaten.erzeugeObjekt(getVerbindung(), datenSatz.getItem(attribut.getName())));
				}
			}
		} else if ("timeout".equals(name)) {
			result = LogischerWert.of(false);
		} else if ("objekt".equals(name)) {
			result = DavDaten.erzeugeObjekt(getVerbindung(), object.getObject());
		} else if ("daten".equals(name)) {
			if (object.getData() != null) {
				result = DavDaten.erzeugeObjekt(getVerbindung(), object.getData());
			} else {
				result = NichtWert.NICHTWERT;
			}
		} else if ("zeit".equals(name)) {
			result = new Zeitstempel(object.getDataTime());
		}

		if (result == null) {
			throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ELEMENT,
					"Der Zugriff auf des Element \"" + name + "\" ist nicht möglich");
		}

		return result;

	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		throw new ObjektZugriffsFehler(UdaFehlerSubtyp.OBJEKT);
	}

}
