/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.uda.data;

import java.io.Serializable;

import de.bsvrz.ibv.uda.uda.UdaKonstante;

/**
 * ein Eintrag für die Historie der Berarbeitung eines Uda-Skripts.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class Historie implements Serializable {
	/**
	 * Versions-ID für die Serialisierung des Objekts.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * der Zeitpunkt der Änderung.
	 */
	private long zeitPunkt = System.currentTimeMillis();

	/**
	 * der Name des Nutzers, der die Änderung vorgenommen hat.
	 */
	private String name = UdaKonstante.STRING_UNBEKANNT;

	/**
	 * die Beschreibung des Grunds für die vorgenommenen Änderungen.
	 */
	private String grund = UdaKonstante.STRING_UNBEKANNT;

	/**
	 * Standardkonstruktor.<br>
	 * Der Konstruktor wird nur intern beim Zurücklesen der Daten aus den auf
	 * der Festplatte abgelegten Informationen verwendet.
	 */
	Historie() {
		this(1L, "", "");
	}

	/**
	 * Konstruktor.<br>
	 * Erzeugt einen Eintrag für die Historie der Änderung eines Uda-Skripts.
	 *
	 * @param zeitPunkt
	 *            der Zeitpunkt der Änderung
	 * @param name
	 *            der Name des Nutzers, der die Änderung vorgenommen hat.
	 * @param grund
	 *            die Beschreibung des Änderungsgrundes.
	 */
	public Historie(final long zeitPunkt, final String name, final String grund) {
		super();
		this.zeitPunkt = zeitPunkt;
		this.name = name;
		this.grund = grund;
	}

	/**
	 * liefert die Beschreibung des Grundes für eine Änderung.
	 *
	 * @return grund die Beschreibung
	 */
	public String getGrund() {
		return grund;
	}

	/**
	 * liefert den Name des Nutzers, der die Änderung vorgenommen hat.
	 *
	 * @return name den Name
	 */
	public String getName() {
		return name;
	}

	/**
	 * liefert den Zeitpunkt für die Änderung in Millisekunden seit dem 1.1.1970
	 * 0 Uhr GMT.
	 *
	 * @return zeitPunkt der Zeitpunkt
	 */
	public long getZeitPunkt() {
		return zeitPunkt;
	}
}
