/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.container;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * ein Ausdruck, der eine Mehrfachauswahl in einem Uda-Container beschreibt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ContainerMehrfachSelektionsSymbol implements Ausdruck {

	/**
	 * der Ausdruck, der das Containerobjekt beschreibt, aus dem die Auswahl der
	 * Elemente erfolgen soll.
	 */
	private final Ausdruck objektAusdruck;

	/**
	 * der Ausdruck, der das Auswahlkriterium beschreibt.
	 */
	private final Ausdruck selektionsAusdruck;

	/**
	 * Konstruktor.<br>
	 *
	 * @param objektAusdruck
	 *            der Ausdruck zur Beschreibung des Objekts
	 * @param selektionsAusdruck
	 *            der Ausdruck zur Auswahl des Elements innerhalb des Objekts
	 */
	public ContainerMehrfachSelektionsSymbol(final Ausdruck objektAusdruck, final Ausdruck selektionsAusdruck) {
		this.objektAusdruck = objektAusdruck;
		this.selektionsAusdruck = selektionsAusdruck;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		Object result = null;
		final Object object = objektAusdruck.interpret(kontext);
		if (object instanceof Container) {
			final Object indexObject = selektionsAusdruck.interpret(kontext);
			if (indexObject instanceof SelektionsBedingung) {
				if (object instanceof UdaContainer) {
					final SelektionsBedingung selektion = (SelektionsBedingung) indexObject;
					result = selektion.select((UdaContainer) object, false);
				}
			}
		}

		return result;
	}

	/**
	 * setzt die Daten eines Objekts.<br>
	 * Die Funktion setzt die Daten des Objekts, das durch den
	 * {@link #objektAusdruck} und den {@link #selektionsAusdruck} beschrieben
	 * ist auf den übergebenen Wert.
	 *
	 * @param kontext
	 *            der Kontext für die Ausführung der Operation
	 * @param daten
	 *            die zu setzenden Daten
	 */
	public void set(final Kontext kontext, final Object daten) {
		final Object object = objektAusdruck.interpret(kontext);
		if (object instanceof Container) {
			final Object indexObject = selektionsAusdruck.interpret(kontext);
			if (indexObject instanceof Number) {
				final int index = ((Number) indexObject).intValue();
				((Container) object).setContainerElement(index, daten);
			}
		}
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(objektAusdruck);
		liste.add(selektionsAusdruck);
		return liste;
	}
}
