/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.verwaltung.protokoll;

import de.bsvrz.ibv.uda.uda.data.SkriptLauf;

/**
 * beschreibt die Protokollierung der Ausgabedaten eines Uda-Skripts auf der
 * Konsole (Standardausgabe).
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class KonsoleProtokoll extends Protokoll {

	/**
	 * Versions-ID für die Serialisierung.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Standardkonstruktor.
	 */
	public KonsoleProtokoll() {
		super(ProtokollTyp.KONSOLE);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion gibt die empfangene Ausgabedaten über die Standardausgabe
	 * aus.
	 *
	 * @see de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll#ausgeben(de.bsvrz.ibv.uda.verwaltung.protokoll.ProtokollEintrag)
	 */
	@Override
	public void ausgeben(final ProtokollEintrag text) {
		System.out.println(text.getText());
	}

	/**
	 * {@inheritDoc}.<br>
	 * Eine Initialisierung des Ausgabekanal ist für diesen Protokolltyp nicht
	 * erforderlich. Die Funktion führt daher keine Operationen aus.
	 *
	 * @see de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll#initialisiere(de.bsvrz.ibv.uda.uda.data.SkriptLauf)
	 */
	@Override
	public void initialisiere(final SkriptLauf lauf) {
		// keine Operationen erforderlich

	}

	/**
	 * {@inheritDoc}.<br>
	 * Ein Abschliessen der Ausgabekanäle ist für diesen Protokolltyp nicht
	 * erforderlich, die Funktion führt daher keine Operationen aus.
	 *
	 * @see de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll#abschliessen()
	 */
	@Override
	public void abschliessen() {
		// keine Operationen erforderlich
	}
}
