/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.fuzzy;

import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * Definition der Operatoren zur Ausführung von Fuzzy-Operationen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public final class UdaFuzzyOperators {

	/**
	 * erzeugt einen liguistischen Term.
	 */
	public static final Operator LING_TERM_KONSTRUKTOR = Operator.getOperator("LinguistischerTerm");

	/**
	 * erzeugt eine liguistische Variable.
	 */
	public static final Operator LING_VAR_KONSTRUKTOR = Operator.getOperator("LinguistischeVariable");

	/**
	 * ermittelt einen Fuzzywert.
	 */
	public static final Operator FUZZYFIZIERE = Operator.getOperator("fuzzyfiziere");

	/**
	 * defuzzyfiziert einen Fuzzywert.
	 */
	public static final Operator DEFUZZYFIZIERE = Operator.getOperator("defuzzyfiziere");

	/**
	 * erzeugt einen "leeren" Fuzzywert.
	 */
	public static final Operator FUZZYWERT = Operator.getOperator("fuzzywert");

	/**
	 * ermittelt die Zugehörigkeit.
	 */
	public static final Operator ZUGEHOERIGKEIT = Operator.getOperator("zugehörigkeit");

	/**
	 * ermittelt das Maximum.
	 */
	public static final Operator MAXIMUM = Operator.getOperator("max");

	/**
	 * ermittelt die algebraische Summe.
	 */
	public static final Operator ALGSUMME = Operator.getOperator("algSumme");

	/**
	 * ermittelt den Komplementärwert.
	 */
	public static final Operator KOMPLEMENT = Operator.getOperator("komplement");

	/**
	 * ermittelt den Gamma-Wert.
	 */
	public static final Operator GAMMA = Operator.getOperator("gamma");

	/**
	 * ermittelt das Minimum.
	 */
	public static final Operator MINIMUM = Operator.getOperator("min");

	/**
	 * ermittelt das algebraische Produkt.
	 */
	public static final Operator ALGPRODUKT = Operator.getOperator("algProdukt");

	/**
	 * die Liste der verfügbaren Operatoren.
	 */
	public static final Operator[] OPERATOREN = { UdaFuzzyOperators.LING_TERM_KONSTRUKTOR,
			UdaFuzzyOperators.LING_VAR_KONSTRUKTOR, UdaFuzzyOperators.FUZZYFIZIERE, UdaFuzzyOperators.DEFUZZYFIZIERE,
			UdaFuzzyOperators.FUZZYWERT, UdaFuzzyOperators.ZUGEHOERIGKEIT, UdaFuzzyOperators.MAXIMUM,
			UdaFuzzyOperators.MINIMUM, UdaFuzzyOperators.ALGSUMME, UdaFuzzyOperators.ALGPRODUKT,
			UdaFuzzyOperators.KOMPLEMENT, UdaFuzzyOperators.GAMMA };

	/** privater Konstruktor für Utility-Klasse. */
	private UdaFuzzyOperators() {
		super();
	}

}
