/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.fuzzy;

import de.bsvrz.iav.fuzzylib.fuzzylib.*;
import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.container.Feld;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;

import java.util.*;

/**
 * Repräsentiert eine linguistische Variable innerhalb eines UdaInterpreter.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class UdaLinguistischeVariable implements Struktur, Kopierbar {

	private LinguistischeVariable linguistischeVariable;
	private List<UdaLinguistischerTerm> terme = new ArrayList<>();

	/**
	 * Konstruktor.<br>
	 *
	 * @param name
	 *            der Name der Variable
	 * @param minimum
	 *            der untere Grenzwert des Wertebereiches
	 * @param maximum
	 *            der obere Grenzwert des Wertebereiches
	 */
	public UdaLinguistischeVariable(final String name, final double minimum, final double maximum) {
		linguistischeVariable = new LinguistischeVariable(name, new Wertebereich(minimum, maximum));
	}

	public LinguistischeVariable getLinguistischeVariable() {
		return linguistischeVariable;
	}

	public void addLinguistischerTerm(UdaLinguistischerTerm term) {
		terme.add(term);
		List<FuzzySet> fuzzySets = new ArrayList<>(linguistischeVariable.getFuzzySets());
		fuzzySets.add(term.getFuzzySet());
		linguistischeVariable = new LinguistischeVariable(
				linguistischeVariable.getName(),
				linguistischeVariable.getWertebereich(),
				fuzzySets
		);
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public UdaLinguistischeVariable erzeugeKopie() {
		return new UdaLinguistischeVariable(linguistischeVariable.getName(), linguistischeVariable.getWertebereich().getMinimum(), linguistischeVariable.getWertebereich().getMaximum());
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#getStrukturElement(java.lang.String)
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object result = null;
		if ("name".equals(name)) {
			result = linguistischeVariable.getName();
		} else if ("untereGrenze".equals(name)) {
			result = linguistischeVariable.getWertebereich().getMinimum();
		} else if ("obereGrenze".equals(name)) {
			result = linguistischeVariable.getWertebereich().getMaximum();
		} else if ("terme".equals(name)) {
			result = new Feld();
			for (final UdaLinguistischerTerm term : terme) {
				((Feld) result).einfuegen(0, term);
			}
		}
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		throw new InterpreterException("Ein Element einer Linguistsichen Variable kann nicht direkt gesetzt werden");
	}

	@Override
	public String toString() {
		return linguistischeVariable.toString();
	}

}
