/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.fuzzy;

import de.bsvrz.iav.fuzzylib.fuzzylib.*;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;

/**
 * Repräsentiert einen Fuzzyterm innerhalb eines UdaInterpreter.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class UdaTerm implements Struktur {

	/**
	 * der zu Grunde liegende Term.
	 */
	private Term term;

	/**
	 * Konstruktor.<br>
	 *
	 * @param term
	 *            der zu Grunde liegende {@link Term}
	 */
	public UdaTerm(final Term term) {
		this.term = term;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#getStrukturElement(java.lang.String)
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		if ("name".equals(name)) {
			return term.getName();
		} else if ("zugehörigkeit".equals(name)) {
			return term.getZugehoerigkeit().getWert();
		}
		throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ARGUMENTE,
				"Strukturelement \"" + name + "\" nicht im Term enthalten");
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		if ("zugehörigkeit".equals(name)) {
			if (daten instanceof Number) {
				term = term.mitZugehoerigkeit(Zugehoerigkeit.von(((Number) daten).doubleValue()));
			}
		}
	}

	@Override
	public String toString() {
		return term.toString();
	}

}
